;---------------------------------------------------------------------------
;
; Beat Player by Paul Slocum
;
;---------------------------------------------------------------------------

	processor 6502	
	include vcs.h	


;---------------------------------------------------------------------------
; Constants
;---------------------------------------------------------------------------
SETUPDELAY equ #255


;---------------------------------------------------------------------------
; RAM Variables 
;---------------------------------------------------------------------------
frame equ $80

temp equ frame + 1

; 16 bit temp
temp16L equ temp + 1
temp16H equ temp16L + 1

; Initialize Flag
init equ temp16H + 1

; Text engine variables
grfxBuffer equ init + 1 ; 24 bytes

; Array of text pointers to be displayed
textPointer equ grfxBuffer + 24	; 22 bytes

scanSec equ textPointer + 22
EOTflag equ scanSec + 1

backColor equ EOTflag + 1

; Variable for reading Keyboards
keyRead equ backColor + 1

note1 equ keyRead + 1
note2 equ note1 + 1

vol1 equ note2 + 1
vol2 equ vol1 + 1

atten1 equ vol2 + 1
atten2 equ atten1 + 1

soundTypeL equ atten2 + 1
soundTypeR equ soundTypeL + 1

sound1 equ soundTypeR + 1
sound2 equ sound1 + 1
tempSoundL equ sound2 + 1
tempSoundR equ tempSoundL + 1

arpOn equ tempSoundR + 1

beatBox1 equ arpOn + 1
beatBox2 equ beatBox1 + 1
beatLock equ beatBox2 + 1

adsrcount equ beatLock + 1
adsrSetting equ adsrcount + 1

adsrLevel1 equ adsrSetting + 1
adsrLevel2 equ adsrLevel1 + 1
lastNote1 equ adsrLevel2 + 1
lastNote2 equ lastNote1 + 1
lastSound1 equ lastNote2 + 1
lastSound2 equ lastSound1 + 1

; Metrenome stuff
tick equ lastSound2 + 1 ;*** (may need for tempo count display)
beat equ tick + 1
tempoCount equ beat + 1

; Color cycle counter
colorCycle equ tempoCount + 1

arpShort equ colorCycle + 1 ;***

setupMode equ arpShort + 1
lastSetupKey equ setupMode + 1

tempo equ lastSetupKey + 1

; Keyboard key buffers
keys1 equ tempo + 1
keypad1 equ keys1 + 1	; 4 bytes
keys2 equ keypad1 + 4
keypad2 equ keys2 + 1	; 4 bytes
tempKeys1 equ keypad2 + 4
tempKeypad1 equ tempKeys1 + 1 ; 4 bytes
tempKeys2 equ tempKeypad1 + 4
tempKeypad2 equ tempKeys2 + 1 ; 4 bytes

; 106 bytes 1-21-02


;---------------------------------------------------------------------------
; Text Macros
;---------------------------------------------------------------------------
; a few macros for storing text in a reasonably easy to read format
;---------------------------------------------------------------------------

	mac off
	dc.b <#{0}
	endm

	mac wordOff
	dc.b #<{1}
	dc.b #>{1}
	endm
	
	mac mapRow
	wordOff {1}
	wordOff {2}	
	wordOff {3}
	wordOff {4}
	wordOff {5}
	endm

	mac EOL
	dc.b #$FF
	endm

	mac EOT
	dc.b #$FE
	endm

	mac line
	;must supply 12 characters, though 0's will not be assembled
	;------------------------------------------------------------
	if {1} != 0
	 off {1}
	endif
	if {2} != 0
	 off {2}
	endif
	if {3} != 0
	 off {3}
	endif
	if {4} != 0
	 off {4}
	endif
	if {5} != 0
	 off {5}
	endif
	if {6} != 0
	 off {6}
	endif
	if {7} != 0
	 off {7}
	endif
	if {8} != 0
	 off {8}
	endif
	if {9} != 0
	 off {9}
	endif
	if {10} != 0
	 off {10}
	endif
	if {11} != 0
	 off {11}
	endif
	if {12} != 0
	 off {12}
	endif
	EOL
	endm
	
	mac blank
	 line #0,#0,#0,#0,#0,#0,#0,#0,#0,#0,#0,#0
	endm



;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
	org $F000
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------



;---------------------------------------------------------------------------
; Font Data
;---------------------------------------------------------------------------
; Include the font data here so it is aligned on a page
; boundary without wasting any space
;---------------------------------------------------------------------------
	include snthfont.h		;just change the filename to use a different font...


;--------------------------------------------------------------------------
; Println
;--------------------------------------------------------------------------
; Most of this subroutine is ripped from Stellar Trak.  Feel free to add
; your own comments--it's pretty straightforward ;)  and very touchy.  Won't
; work if a page boundary appears at certain places, due to excruciating
; cycle dependency, so if modifying the source, you might need an ALIGN
; pseudo-op to correct flubbery text displays.  ALIGN 256 will always work,
; but maybe ALIGN 128, ALIGN 64, etc. will also work, and not waste as many
; bytes.
;--------------------------------------------------------------------------
	align 256
println
	ldx #4
	sta WSYNC
pause
	nop
	dex	
	bne pause
	lda temp
	lda temp	
		
	sta HMCLR
	ldx #$90
	ldy #6	;FONTHEIGHT
	lda frame
	and #$01
	beq oddFrame
	jmp evenFrame

print1	
	sta GRP1
	lda (grfxBuffer + $8),y
	sta GRP0
	lda (grfxBuffer + $C),y
	stx HMP0
	stx HMP1
	sta GRP1
	lda (grfxBuffer + $10),y
	sta GRP0
	lda (grfxBuffer + $14),y
	sta GRP1
	sta GRP0
evenFrame	
	dey
	bmi wrapEven
	lda (grfxBuffer + $2),y
	lsr
	sta GRP0
	lda (grfxBuffer +6),y
	lsr
	sta.w $001C
	sta HMOVE
	lda (grfxBuffer + $A),y
	lsr
	sta GRP0
	lda (grfxBuffer + $12),y
	lsr
	sta temp
	lda (grfxBuffer + $E),y
	lsr
	sta GRP1
	lda temp
	sta GRP0
	lda (grfxBuffer + $16),y
	lsr
	sta GRP1

oddFrame
	sta GRP0
	lda #$70
	sta HMP0
	sta HMP1
	dey
	bmi wrapOdd
	lda (grfxBuffer),y
	sta GRP0
	lda (grfxBuffer + $4),y
	sta HMOVE
	jmp print1

wrapEven
	stx HMP0
	stx HMP1
	sta WSYNC
	sta HMOVE
	jmp print2

wrapOdd
	sta WSYNC
	sta temp
	sta temp

print2
	lda #$00
	sta GRP0
	sta GRP1
	sta GRP0

	rts ; println


				
;---------------------------------------------------------------------------
; Start of Program
;---------------------------------------------------------------------------
; Clear memory, locate character graphics positions and such,
; initialize key memory values, start GameLoop.
;-------------------------------------------------------------
Start
	sei  	
	cld  		
	ldx #$FF
	txs  		
	lda #0
clear   
	sta 0,x
	dex
	bne clear	

;---------------------------------------------------------------------------
; Text Engine Graphics Setup
;---------------------------------------------------------------------------
; use cycle counting to get the GRP registers configured and located
; in exactly the right spot
;---------------------------------------------------------------------------
	sta WSYNC	;newline
	lda #54		;triple copy sprites, spaced wide
	sta NUSIZ0	;for both player graphic registers
	sta NUSIZ1	
	lda #$31	;draw leftmost and rightmost 2 playfield bits: reflect
	sta PF0		;this turns left/right edges black
	sta CTRLPF	;this makes playfield reflective
	nop		;wait 2 cycles, we need them get our RESP0/1 right
	sta VDELP0	;turn on vertical delay for both players
	sta VDELP1
	sta RESP0	;mark our left margin here in P0
	lda #$D0	;setup to shift over 3 pixels to the right
	sta RESP1	;set P1
	sta HMP0	;prep P0 for his HMOVE
	lda #$C0	;setup to shift P1 4 pixels to the right
	sta HMP1	;prep P1 for his HMOVE

	sta WSYNC	;newline
	sta HMOVE	;anchor them down



;---------------------------------------------------------------------------
; Font Setup
;---------------------------------------------------------------------------
;the entire font is limited to one page, so we can hard code that page
;value into the MSB position of our graphics buffer
;-----------------------------------------------------
	ldx #23		;graphics buffer is 24 bytes
	lda #>_space	;load up the page value for the font
grOff
	sta grfxBuffer,x	;stuff it, won't ever change
	dex		;back up two bytes
	dex
	bpl grOff	;and loop


;---------------------------------------------------------------------------
; Initialize variables
;---------------------------------------------------------------------------
	lda #0			
	sta colorCycle
	sta keyRead
	sta keypad1
	sta keypad2
	sta tempKeypad1
	sta tempKeypad2
	
	sta beat
	sta tempoCount

	sta arpOn
	sta arpShort

	sta tick
	sta setupMode

	sta adsrcount
	sta adsrSetting

	sta adsrLevel1
	sta adsrLevel2

	lda #1
	sta beatBox1
	sta beatBox2

	lda #3
	sta tempo

	; Set initialization flag.
	; This stays set until the first frame is drawn.
	lda #255
	sta init
	sta lastSetupKey

	sta lastNote1
	sta lastNote2

	lda #$0f	;set text color to white
	sta COLUP0
	sta COLUP1	



;--------------------------------------------------------------------------
; GameLoop
;--------------------------------------------------------------------------
GameLoop
	jsr VSync 	;start vertical retrace

	inc frame	;we count frames for alternating graphics displays

	jsr VBlank    	; spare time during screen blank
	jsr Picture		; draw one screen
	jsr overscan	; do overscan

	; Clear init flag
	lda #0
	sta init

	jmp GameLoop    ;back to top


;--------------------------------------------------------------------------
; VSync
;--------------------------------------------------------------------------
VSync
	lda #2		;bit 1 needs to be 1 to start retrace
	sta VSYNC	;start retrace
	sta WSYNC 	;wait a few lines
	sta WSYNC 
	lda #44		;prepare timer to exit blank period (44)
	sta TIM64T	;turn it on
	sta WSYNC 	;wait one more
	sta VSYNC 	;done with retrace, write 0 to bit 1

	rts ; VSync


;--------------------------------------------------------------------------
; Voice Manager
;--------------------------------------------------------------------------
voiceManager

	ldx #0
	stx beatBox1
	inx
	stx beatBox2

	lda SWCHB
	and #%01000000
	bne beat1On

	ldx #3
	stx beatBox1

beat1On

	lda SWCHB
	and #%10000000
	bne beat2On

	ldx #3
	stx beatBox2 

beat2On

	lda #2	; Two voices available
	jsr beatBox
	
	rts

;--------------------------------------------------------------------------
; Tempo
;--------------------------------------------------------------------------
; Generates tempo based on set tempo
;--------------------------------------------------------------------------
handleTempo

	lda #3
	sta tempo

	lda SWCHB
	and #%00001000
	bne fastTempo

	lda #6
	sta tempo

fastTempo

	lda tempoCount
	cmp #15
	bmi notReset

	lda #0
	sta tempoCount

notReset
	



	lda #0
	sta tick
	ldx tempoCount
	inx
	stx tempoCount
	
	lda tempo
	clc
	adc #4
	cmp tempoCount
	bne quitTempo

	lda #0
	sta tempoCount

	lda #255
	sta tick
	
	ldx beat
	inx
	stx beat
	cpx #32
	bne quitTempo

	lda #0
	sta beat

quitTempo

	rts ; tempo


;--------------------------------------------------------------------------
; Extract Beat Data
;--------------------------------------------------------------------------
; Each byte of beat data contains the frequency and 
; sound type data.  This function separates and decodes them.
;
; The encoding is: the 3 high bits contain the encoded sound
; type and the lower 5 bits contain the freq data.
;
; - ACC must contain beat byte
;
; - ACC will return the freq
; - X will return the sound type
;--------------------------------------------------------------------------
extractBeatData
	cmp #255
	beq noNote
	
	tax

	; Extract freq data and push it
	and #%00011111
	pha

	txa
	lsr
	lsr
	lsr
	lsr
	lsr
	tax
	
	lda soundTypeArray,x

	tax
	pla

	rts

noNote
	lda #255

	rts ; extractBeatData : other rts



;--------------------------------------------------------------------------
; Accent Reader
;--------------------------------------------------------------------------
; Each set of beat data is followed by 4 accept bytes.
; Each bit in order represents the accent (on or off)
; of its corresponding 32nd note.  This function
; returns the attenuation of a note in a beat.
;
; - temp16 must contain an indirect pointer to the beat data
; - Y must contain the current 32nd note
;
; = will return the attenuation in ACC
;--------------------------------------------------------------------------
bitMaskArray
	byte #%10000000
	byte #%01000000
	byte #%00100000
	byte #%00010000
	byte #%00001000
	byte #%00000100
	byte #%00000010
	byte #%00000001

accentReader
	; There are 4 accent bytes.  Get the
	; correct one.
	tya
	pha

	and #%00011000
	lsr
	lsr
	lsr

	; The 4 accent bytes are after the 32 bytes
	; that make up the beat.
	clc
	adc #32
	tay

	; Now set up a mask for the bit
	pla
	and #%00000111
	tax
	
	lda (temp16L),y
	and bitMaskArray,x
	beq noAccent

	; Accent, so don't attenuate
	lda #0

	rts

noAccent

	; No accent, so attenuate a bit
	lda #10

	rts ; accentReader




;--------------------------------------------------------------------------
; Beat Box
;--------------------------------------------------------------------------
; Plays up to two pre-programmed beats simlutaneously.
; - Assumes use of right keypad only for beat box
; - ACC must contain the number of voices available (1 or 2)
; 
; If two oscillators are available, the other oscillator
; will be used for a second beat if one is selected.
;--------------------------------------------------------------------------
beatBox

	;DEBUG
	;lda #3
	;sta beatBox1
	;sta beatBox2
	
; Play beat 1
;--------------------------------------------------------------------------
	; Get address of selected beat
	lda beatBox1
	asl
	tax
	lda beatArray,x
	sta temp16L
	inx
	lda beatArray,x
	sta temp16H

	; The variable, beat, contains the 32nd note
	; that the beat is currently on.
	ldy beat

	lda (temp16L),y
	jsr extractBeatData

	sta note2
	stx sound2	

	jsr accentReader
	sta atten2


; Play beat 2
;--------------------------------------------------------------------------

playBeat2
	; Get address of selected beat
	lda beatBox2
	asl
	tax
	lda beatArray,x
	sta temp16L
	inx
	lda beatArray,x
	sta temp16H

	; The variable, beat, contains the 32nd note
	; that the beat is currently on.
	ldy beat

	lda (temp16L),y
	jsr extractBeatData

	; If both voices are available, play
	; beat2 on osc1

	sta note1
	stx sound1	

	ldy beat
	jsr accentReader
	sta atten1

beatBoxEnd

	;DEBUG
	;lda #255
	;sta note1
	;sta note2

	rts ; beatBox





;--------------------------------------------------------------------------
; VBlank
;--------------------------------------------------------------------------
; Handle user input and display setup during the VBLANK period
;--------------------------------------------------------------------------
VBlank
	jsr handleTempo
	jsr voiceManager
	jsr calcVolume

	rts ; VBlank

;--------------------------------------------------------------------------
; Overscan
;--------------------------------------------------------------------------
; I ran out of time in VBlank, so I'm doing some things during overscan
;--------------------------------------------------------------------------
overscan

	sta WSYNC	
	lda #33		; Use the timer to make sure overscan takes (34)
	sta TIM64T	; 30 scan lines.  29 scan lines * 76 = 2204 / 64 = 34.4

	jsr handleTempo
	jsr voiceManager
	jsr calcVolume
	jsr notePlayer

endOS	
	lda INTIM	; We finished, but wait for timer
	bne endOS	; by looping till zero
	
	sta WSYNC	; End last scanline

	lda #$82
	sta VBLANK
	lda #$02
	sta VBLANK

	rts	; overscan


;--------------------------------------------------------------------------
; Calculate Volume
;--------------------------------------------------------------------------
calcVolume

	lda #15
	sta vol1
	sta vol2

	; Handle release
	;--------------------------------------------------------------------------

	lda note1
	cmp #255
	bne rel2

	ldx #0
	stx vol1

rel2

	lda note2
	cmp #255
	bne endRelease

	ldx #0
	stx vol2

endRelease

; Handle attenuation
;--------------------------------------------------------------------------
	; Attenuation 1	
	lda vol1
	beq handleAtten2
	sbc atten1	
	bcs notZero1

	lda #1

notZero1
	sta vol1

	; Handle Attenuation

handleAtten2
	; Attenuation 2
	lda vol2
	beq endAtten
	sbc atten2
	bcs notZero2

	lda #1

notZero2
	sta vol2

endAtten

	lda #0
	sta atten1
	sta atten2


	rts ; calcVolume


;--------------------------------------------------------------------------
; Note Player
;--------------------------------------------------------------------------
; Actually plays assigned notes using assigned voices
;--------------------------------------------------------------------------
notePlayer

	; Sound type
	ldx sound1
	ldy sound2
	stx AUDC0
	sty AUDC1

	; Frequency
	ldx note1
	ldy note2
	stx AUDF0
	sty AUDF1

	; Volume
	ldx vol1
	ldy vol2
	stx AUDV0
	sty AUDV1
	
	rts ; notePlayer



;--------------------------------------------------------------------------
; Draw TV Pictures
;--------------------------------------------------------------------------
; Here is where we step down the screen drawing everything.
;
; Note that temp16 is used in here as a text pointer, so don't use
; it for anything else during the loop.
;--------------------------------------------------------------------------
;	align 256
Picture
	lda #0
	sta COLUBK

	lda #11		;setup counter for rows of text
	sta scanSec	;store in zero page memory variable

	ldy #0		;Y is used to track progress through this screen's text
	sty EOTflag

pictureLoop
	lda INTIM	;check timer for end of VBLANK period
	bne pictureLoop	;loop until it reaches 0

	;sta WSYNC	;newline (1)
	lda #$80
	sta VBLANK  	;end screen blank

ScanLoop	
;---------------------------------------------------------------------------

	; Around the middle of the screen, the notePlayer is called.
	; This must occur on a line where the white divider line
	; is not drawn (on the Status or Setup screen)
	cpy #2
	bne skipNotePlayer
	jsr notePlayer

skipNotePlayer

	lda #64		; A variable amount of text will be processed, so stay....

	ldx backColor

	sta WSYNC
	jmp timer


; Timer
;--------------------------------------------------------------------------

	align 256
timer

	sta  TIM8T	;....in sync by timing out after finished (7 scanlines)

	stx COLUBK

	ldx #22		;write 12 spaces to text buffer to wipe out what's there
	lda #<_space	; from previous row/frame/etc.
blankLine
	sta grfxBuffer,x	;grfxBuffer is our 24 byte, 16 bit list of chars
	dex	
	dex
	bpl blankLine	;fill it
	lda EOTflag
	bne eol
	
	ldx #0		; Will load from left to right.
	ldy #0		; Each line is referenced individually.
nextCol
	lda textData,y	; Indirect/indexed - pointer to a section of text
	iny			; next char
	cmp #$FF	; FF is EOL
	beq eol		; done with a line?
	cmp #$FE
	bne noEOT
	lda #1
	sta EOTflag
	jmp eol
noEOT	
    sta grfxBuffer,x	; No, stuff this char into the buffer
	inx	
	inx
	jmp nextCol	; Just keep going--a screen w/o 10 $FF's will screw this up

eol	
	lda INTIM	; We finished, but wait for timer
	bne eol		; by looping till zero
	sta WSYNC	; end current line

	tya			; Need to save Y by putting into accumulator
	pha			; then onto stack.
	jsr println	; Print this row via subroutine. (6 scanlines)
	pla			; Pull Y off stack
	tay			; and put back.

	dec scanSec	; next row to print
	beq quitScanLoop; loop until all 11 rows of text have been displayed

	jmp ScanLoop

quitScanLoop

	sta WSYNC
	sta WSYNC
	sta WSYNC

	lda #$82
	sta VBLANK	; Finished a screen, blank the beam.
endScreen

	rts	; Picture

textData
	line B,E,A,T,_sp,P,L,A,Y,E,R,0,0

	align 256
beatArray
	word beat1, beat2, beat3, beat3

	include beatdata.h

beat3
	byte 255,255,255,255
	byte 255,255,255,255
	byte 255,255,255,255
	byte 255,255,255,255

	byte 255,255,255,255
	byte 255,255,255,255
	byte 255,255,255,255
	byte 255,255,255,255

	byte %11111111, %11111111, %11111111, %11111111


soundData1
	byte 29,26,23
	byte 19,17,15
	byte 14,11,9
	byte 4,2,1

soundTypeArray
	byte 4,6,7,8
	byte 15,12,1,3


;---------------------------------------------------------------------------
; Program Startup Vector
;---------------------------------------------------------------------------
	org $FFFC
	.word Start
	.word Start

